@@ -4,19 +4,31 @@ Run terminal command 'graph-onedrive --help' or 'python -m graph-onedrive --help
 from __future__ import annotations
 
 import argparse
-import json
 import logging
 import os
 from datetime import datetime
+from pathlib import Path
 from typing import Sequence
 
 from graph_onedrive.__init__ import __version__
+from graph_onedrive._config import CONFIG_EXTS
+from graph_onedrive._config import dump_config
+from graph_onedrive._config import load_config
 from graph_onedrive._main import OneDrive
 
 
-CONFIG_EXT = ".json"
-CONFIG_DEF_FILE = "config" + CONFIG_EXT
-CONFIG_DEF_KEY = "onedrive"
+CONFIG_DEFAULT_FILENAME = "config"
+CONFIG_DEFAULT_KEY = "onedrive"
+
+
+# Set logger
+logger = logging.getLogger(__name__)
+package_logger = logging.getLogger(__package__)
+package_logger.propagate = True
+handler = logging.StreamHandler()
+formatter = logging.Formatter("[%(levelname)s] %(message)s")
+handler.setFormatter(formatter)
+package_logger.addHandler(handler)
 
 
 def main(argv: Sequence[str] | None = None) -> int:
@@ -57,7 +69,7 @@ def main(argv: Sequence[str] | None = None) -> int:
         action="store",
         type=str,
         metavar="PATH",
-        help="optional path to config json file",
+        help="optional path to config json or yaml file",
     )
     parser.add_argument(
         "-k",
@@ -71,7 +83,7 @@ def main(argv: Sequence[str] | None = None) -> int:
         "--log",
         action="count",
         default=0,
-        help="displays logs (level=INFO), use -ll for level=DEBUG",
+        help="displays INFO logs, use -ll for DEBUG logs",
     )
     # Parse arguments, using function input args when given for tests
     args = parser.parse_args(argv)
@@ -80,17 +92,26 @@ def main(argv: Sequence[str] | None = None) -> int:
     if not (args.configure or args.authenticate or args.instance):
         parser.error("No action provided, use --help for details")
 
-    if args.file and not args.file.endswith(CONFIG_EXT):
-        parser.error(f"--file path must have {CONFIG_EXT} extension")
+    if args.file and not args.file.endswith(CONFIG_EXTS):
+        if args.file.endswith((".yaml",)):
+            parser.error(f"--file path was to yaml file but PyYAML is not installed")
+        elif args.file.endswith((".toml",)):
+            parser.error(f"--file path was to toml file but TOML is not installed")
+        else:
+            parser.error(
+                f"--file path must have {' or '.join([i for i in CONFIG_EXTS])} extension"
+            )
 
     if args.key and args.key == "":
         parser.error("--key provided can not be blank")
 
-    # Configure logger
+    # Configure logger level
     if args.log == 1:
-        logging.basicConfig(level=logging.INFO)
+        package_logger.setLevel(logging.INFO)
+        logger.info("Logging enabled with level=INFO")
     elif args.log == 2:
-        logging.basicConfig(level=logging.DEBUG)
+        package_logger.setLevel(logging.DEBUG)
+        logger.info("Logging enabled with level=DEBUG")
 
     # Call action functions
     if args.configure:
@@ -107,45 +128,54 @@ def main(argv: Sequence[str] | None = None) -> int:
 def config(config_path: str | None = None, config_key: str | None = None) -> None:
     """Create a configuration file."""
 
-    # Set the export directory
+    if config_path and not config_path.endswith(CONFIG_EXTS):
+        raise ValueError(
+            f"config_path expected extension {' or '.join([i for i in CONFIG_EXTS])}"
+        )
+
+    # Set the export directory when not set as an argument
     if not config_path:
+
         if (
-            input(f"Save {CONFIG_DEF_FILE} in current working directory [Y/n]: ")
+            input(
+                f"Save as {CONFIG_DEFAULT_FILENAME}.json in current working directory? [Y/n]: "
+            )
             .strip()
             .lower()
-            == "n"
+            != "n"
         ):
-            config_path = input("Path to save config (including file name): ").strip()
-        else:
-            config_path = os.path.join(os.getcwd(), CONFIG_DEF_FILE)
+            config_path = os.path.join(os.getcwd(), CONFIG_DEFAULT_FILENAME + ".json")
 
-    if not config_path.endswith(CONFIG_EXT):
-        config_path += CONFIG_EXT
+        else:
+            config_path = ""
+            while not config_path.endswith(CONFIG_EXTS):
+                config_path = input(
+                    f"Path to config file (with extension {' or '.join([i for i in CONFIG_EXTS])}): "
+                ).strip()
 
     # Set config dictionary key
     if not config_key:
         if (
-            input(f"Use config dictionary key default '{CONFIG_DEF_KEY}' [Y/n]: ")
+            input(f"Use config dictionary key default '{CONFIG_DEFAULT_KEY}' [Y/n]: ")
             .strip()
             .lower()
             == "n"
         ):
             config_key = input("Config dictionary key to use: ").strip()
         else:
-            config_key = CONFIG_DEF_KEY
+            config_key = CONFIG_DEFAULT_KEY
 
     # Load the current file if it exists, otherwise create dictionary
     if os.path.isfile(config_path):
-        with open(config_path) as config_file:
-            config = json.load(config_file)
+        config = load_config(config_path, config_key)
         # For safety do not overwrite existing configs
-        if config_key in config:
-            raise SystemExit(
-                f"Error: '{config_key}' already exists in file, overwrite not permitted."
+        if config:
+            logger.error(
+                f"'{config_key}' already exists within {Path(config_path).name}"
             )
-        config[config_key] = {}
+            raise SystemExit()
     else:
-        config = {config_key: {}}
+        config = {}
 
     # Set basic app credentials
     tenant_id = input("tenant: ").strip()
@@ -164,32 +194,28 @@ def config(config_path: str | None = None, config_key: str | None = None) -> Non
         redirect_url = "http://localhost:8080"
 
     # Format the config into the dictionary
-    config[config_key]["tenant_id"] = tenant_id
-    config[config_key]["client_id"] = client_id
-    config[config_key]["client_secret_value"] = client_secret_value
-    config[config_key]["redirect_url"] = redirect_url
-    config[config_key]["refresh_token"] = None
+    config["tenant_id"] = tenant_id
+    config["client_id"] = client_id
+    config["client_secret_value"] = client_secret_value
+    config["redirect_url"] = redirect_url
+    config["refresh_token"] = None
 
     # Save the configuration to config file
-    with open(config_path, "w") as config_file:
-        json.dump(config, config_file, indent=4)
+    dump_config(config, config_path, config_key)
     print(f"Configuration saved to: {config_path}")
 
 
 def authenticate(config_path: str | None = None, config_key: str | None = None) -> None:
     """Authenticate with OneDrive and then save the configuration to file."""
     # Get the config file path
-    config_path, config_key = get_config_file(config_path, config_key)
+    config_path, config_key = _get_config_file(config_path, config_key)
 
-    # Create the instance
-    onedrive = OneDrive.from_json(config_path, config_key)
-
-    # Save the config
-    onedrive.to_json(config_path, config_key)
+    # Create the instance and save the config
+    OneDrive.from_file(config_path, config_key, save_refresh_token=True)
     print(f"Refresh token saved to configuration: {config_path}")
 
 
-def get_config_file(
+def _get_config_file(
     config_path: str | None = None, config_key: str | None = None
 ) -> tuple[str, str]:
     """Sets a config path and key by searching the cwd with assistance from from user."""
@@ -200,7 +226,7 @@ def get_config_file(
         count = 0
         for root, dirs, files in os.walk(cwd_path):
             for file_name in files:
-                if file_name.endswith(CONFIG_EXT):
+                if file_name.endswith(CONFIG_EXTS):
                     if (
                         input(f"Found: {file_name} Use this? [Y/n]: ").strip().lower()
                         != "n"
@@ -218,30 +244,31 @@ def get_config_file(
         if not config_path:
             while True:
                 config_path = input("Path to config file: ").strip()
-                if not config_path.endswith(CONFIG_EXT):
-                    config_path = config_path + CONFIG_EXT
-                if os.path.isfile(config_path):
+                if os.path.isfile(config_path) and config_path.endswith(CONFIG_EXTS):
                     break
                 print("Path could not be validated, please try again.")
+                print("Ensure that the path includes the filename and extension.")
+                print(f"Acceptable file types: {CONFIG_EXTS}")
 
     # Open the config file
-    with open(config_path) as config_file:
-        config = json.load(config_file)
-
-    # Check the config key
-    if not config_key:
-        config_key = CONFIG_DEF_KEY
-        if config_key in config:
+    if config_key:
+        config = load_config(config_path, config_key)
+    else:
+        try:
+            config_key = CONFIG_DEFAULT_KEY
+            config = load_config(config_path, config_key)
             if (
-                input("Config dictionary key 'onedrive' found. Use this key? [Y/n]: ")
+                input(
+                    f"Config dictionary key '{CONFIG_DEFAULT_KEY}' found. Use this key? [Y/n]: "
+                )
                 .strip()
                 .lower()
                 == "n"
             ):
-                config_key = input("Config dictionary key to use: ").strip()
-    while config_key not in config:
-        print(f"Config dictionary key '{config_key}' not found.")
-        config_key = input("Config dictionary key to use: ").strip()
+                raise KeyError()
+        except KeyError:
+            config_key = input("Config dictionary key to use: ").strip()
+            config = load_config(config_path, config_key)
 
     return config_path, config_key
 
@@ -260,12 +287,13 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
     # Create the instance
     if use_config_file:
         # Get the config details
-        config_path_verified, config_key_verified = get_config_file(
+        config_path_verified, config_key_verified = _get_config_file(
             config_path, config_key
         )
 
         # Create session
-        onedrive = OneDrive.from_json(config_path_verified, config_key_verified, True)
+        onedrive = OneDrive.from_file(config_path_verified, config_key_verified, True)
+
     else:
         print("Manual configuration entry:")
         client_id = input("client_id: ").strip()
@@ -281,7 +309,8 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
     Graph-OneDrive cli instance actions:
     u / usage      :  prints the OneDrive usage
     od / onedrive  :  print the OneDrive details
-    li / list      :  list the contents of a folder
+    ls / list      :  list the contents of a folder
+    se / search    :  list items matching a query
     de / detail    :  print metadata of an item
     sl / link      :  create a sharing link for an item
     md / mkdir     :  make a new folder
@@ -302,10 +331,10 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
             if command == "help":
                 print(help_info)
 
-            elif command in ["u", "usage"]:
+            elif command in ("u", "usage"):
                 onedrive.get_usage(verbose=True)
 
-            elif command in ["li", "list"]:
+            elif command in ("ls", "li", "list"):
                 folder_id: str | None = input(
                     "Folder id to look into (enter nothing for root): "
                 ).strip()
@@ -316,7 +345,16 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                     continue
                 onedrive.list_directory(folder_id, verbose=True)
 
-            elif command in ["de", "detail"]:
+            elif command in ("se", "search"):
+                query = input("Search query: ").strip()
+                top = input("Max number of results: ").strip()
+                try:
+                    top_n = int(top)
+                except ValueError:
+                    top_n = 10
+                onedrive.search(query, top_n, verbose=True)
+
+            elif command in ("de", "detail"):
                 item_id = input("Item id or root path starting with /: ").strip()
                 if item_id[0] == "/":
                     onedrive.detail_item_path(item_id, verbose=True)
@@ -364,7 +402,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                 )
                 print(response)
 
-            elif command in ["md", "mkdir"]:
+            elif command in ("md", "mkdir"):
                 parent_folder_id: str | None = input(
                     "Parent folder id (enter nothing for root): "
                 ).strip()
@@ -377,7 +415,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                 response = onedrive.make_folder(folder_name, parent_folder_id)
                 print(response)
 
-            elif command in ["mv", "move"]:
+            elif command in ("mv", "move"):
                 item_id = input("Item id of the file/folder to move: ").strip()
                 new_folder_id = input("New parent folder id: ").strip()
                 if input("Specify a new file name? [y/N]: ").strip().lower() == "y":
@@ -389,7 +427,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                 response = onedrive.move_item(item_id, new_folder_id, new_file_name)
                 print(f"Item {item_id} was moved to {new_folder_id}")
 
-            elif command in ["cp", "copy"]:
+            elif command in ("cp", "copy"):
                 item_id = input("Item id of the file/folder to copy: ").strip()
                 new_folder_id = input("New parent folder id: ").strip()
                 if input("Specify a new name? [y/N]: ").strip().lower() == "y":
@@ -407,19 +445,19 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                     f"Item was copied to folder {new_folder_id} with new item id {response}"
                 )
 
-            elif command in ["rn", "rename"]:
+            elif command in ("rn", "rename"):
                 item_id = input("Item id of the file/folder to rename: ").strip()
                 new_file_name = input("New file name (with extension): ").strip()
                 response = onedrive.rename_item(item_id, new_file_name)
                 print("Item was renamed.")
 
-            elif command in ["rm", "remove", "delete"]:
+            elif command in ("rm", "remove", "delete"):
                 item_id = input("Item id of the file/folder to remove: ").strip()
                 response = onedrive.delete_item(item_id)
                 if response == True:
                     print(f"Item {item_id} was successfully removed.")
 
-            elif command in ["dl", "download"]:
+            elif command in ("dl", "download"):
                 item_id = input("File item id to download: ").strip()
                 if onedrive.is_folder(item_id):
                     print("Item id is a folder. Folders cannot be downloaded.")
@@ -430,7 +468,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                     f"Item was downloaded in the current working directory as {response}"
                 )
 
-            elif command in ["ul", "upload"]:
+            elif command in ("ul", "upload"):
                 file_path = input("Provide full file path: ").strip()
                 if input("Rename file? [y/N]: ").strip().lower() == "y":
                     new_file_name = input(
@@ -451,7 +489,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
                 )
                 print(f"New file item id: {item_id}")
 
-            elif command in ["od", CONFIG_DEF_KEY, "drive"]:
+            elif command in ("od", CONFIG_DEFAULT_KEY, "drive"):
                 print("Drive id:    ", onedrive._drive_id)
                 print("Drive name:  ", onedrive._drive_name)
                 print("Drive type:  ", onedrive._drive_type)
@@ -480,7 +518,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
             elif command == "_refresh":
                 print(onedrive.refresh_token)
 
-            elif command in ["exit", "exit()", "quit", "q", "end"]:
+            elif command in ("exit", "exit()", "quit", "q", "end"):
                 break
 
             else:
@@ -488,7 +526,7 @@ def instance(config_path: str | None = None, config_key: str | None = None) -> N
 
     finally:
         if use_config_file:
-            onedrive.to_json(
+            onedrive.to_file(
                 config_path_verified,
                 config_key_verified,
             )
